# encoding: utf-8
#
# Copyright (c) 2010 Doug Hellmann.  All rights reserved.
#
"""virtualenvwrapper.project
"""

import logging
import os

from virtualenvwrapper.user_scripts import make_hook, run_global, PERMISSIONS

log = logging.getLogger(__name__)

GLOBAL_HOOKS = [
    # mkproject
    ("premkproject",
     "This hook is run after a new project is created "
     "and before it is activated.",
     PERMISSIONS),
    ("postmkproject",
     "This hook is run after a new project is activated.",
     PERMISSIONS),
]


def initialize(args):
    """Set up user hooks
    """
    for filename, comment, permissions in GLOBAL_HOOKS:
        make_hook(os.path.join('$VIRTUALENVWRAPPER_HOOK_DIR', filename),
                  comment, permissions)
    return


def pre_mkproject(args):
    log.debug('pre_mkproject %s', str(args))
    run_global('premkproject', *args)
    return


def post_mkproject_source(args):
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/postmkproject" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/postmkproject"
"""


def post_activate_source(args):
    return """
#
# Change to the project directory
#
[ -f "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME" ] && \
    virtualenvwrapper_cd \
        "$(cat \"$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME\")"
"""



# Test package for virtualenvwrapper tests
from setuptools import setup

version = '1.0'

setup(
    name='testpackage',
    version=version,
    description="Fake package",
    author="Ingeniweb",
    author_email='thomas.desvenain@gmail.com',
    url='http://pypi.python.org/pypi/testpackage/',
    scripts=['testscript.py']
    )



# encoding: utf-8
#
# Copyright (c) 2010 Doug Hellmann.  All rights reserved.
#
"""Plugin to handle hooks in user-defined scripts.
"""

import logging
import os
import re
import stat
import subprocess
import sys


log = logging.getLogger(__name__)

# Are we running under msys
if sys.platform == 'win32' and \
   os.environ.get('OS') == 'Windows_NT' and \
   os.environ.get('MSYSTEM') == 'MINGW32':
    is_msys = True
    script_folder = 'Scripts'
else:
    is_msys = False
    script_folder = 'bin'


def run_script(script_path, *args):
    """Execute a script in a subshell.
    """
    if os.path.exists(script_path):
        cmd = [script_path] + list(args)
        if is_msys:
            cmd = [get_path(os.environ['MSYS_HOME'], 'bin', 'sh.exe')] + cmd
        log.debug('running %s', str(cmd))
        try:
            subprocess.call(cmd)
        except OSError:
            _, msg, _ = sys.exc_info()
            log.error('could not run "%s": %s', script_path, str(msg))
        # log.debug('Returned %s', return_code)
    return


def run_global(script_name, *args):
    """Run a script from $VIRTUALENVWRAPPER_HOOK_DIR.
    """
    script_path = get_path('$VIRTUALENVWRAPPER_HOOK_DIR', script_name)
    run_script(script_path, *args)
    return


PERMISSIONS = (
    stat.S_IRWXU    # read/write/execute, user
    | stat.S_IRGRP  # read, group
    | stat.S_IXGRP  # execute, group
    | stat.S_IROTH  # read, others
    | stat.S_IXOTH  # execute, others
)
PERMISSIONS_SOURCED = PERMISSIONS & ~(
    # remove executable bits for
    stat.S_IXUSR     # ... user
    | stat.S_IXGRP   # ... group
    | stat.S_IXOTH   # ... others
)


GLOBAL_HOOKS = [
    # initialize
    ("initialize",
     "This hook is sourced during the startup phase "
     "when loading virtualenvwrapper.sh.",
     PERMISSIONS_SOURCED),

    # mkvirtualenv
    ("premkvirtualenv",
     "This hook is run after a new virtualenv is created "
     "and before it is activated.\n"
     "# argument: name of new environment",
     PERMISSIONS),
    ("postmkvirtualenv",
     "This hook is sourced after a new virtualenv is activated.",
     PERMISSIONS_SOURCED),

    # cpvirtualenv:
    # precpvirtualenv <old> <new> (run),
    # postcpvirtualenv (sourced)

    # rmvirtualenv
    ("prermvirtualenv",
     "This hook is run before a virtualenv is deleted.\n"
     "# argument: full path to environment directory",
     PERMISSIONS),
    ("postrmvirtualenv",
     "This hook is run after a virtualenv is deleted.\n"
     "# argument: full path to environment directory",
     PERMISSIONS),

    # deactivate
    ("predeactivate",
     "This hook is sourced before every virtualenv is deactivated.",
     PERMISSIONS_SOURCED),
    ("postdeactivate",
     "This hook is sourced after every virtualenv is deactivated.",
     PERMISSIONS_SOURCED),

    # activate
    ("preactivate",
     "This hook is run before every virtualenv is activated.\n"
     "# argument: environment name",
     PERMISSIONS),
    ("postactivate",
     "This hook is sourced after every virtualenv is activated.",
     PERMISSIONS_SOURCED),

    # mkproject:
    # premkproject <new project name> (run),
    # postmkproject (sourced)

    # get_env_details
    ("get_env_details",
     "This hook is run when the list of virtualenvs is printed "
     "so each name can include details.\n"
     "# argument: environment name",
     PERMISSIONS),
]


LOCAL_HOOKS = [
    # deactivate
    ("predeactivate",
     "This hook is sourced before this virtualenv is deactivated.",
     PERMISSIONS_SOURCED),
    ("postdeactivate",
     "This hook is sourced after this virtualenv is deactivated.",
     PERMISSIONS_SOURCED),

    # activate
    ("preactivate",
     "This hook is run before this virtualenv is activated.",
     PERMISSIONS),
    ("postactivate",
     "This hook is sourced after this virtualenv is activated.",
     PERMISSIONS_SOURCED),

    # get_env_details
    ("get_env_details",
     "This hook is run when the list of virtualenvs is printed "
     "in 'long' mode so each name can include details.\n"
     "# argument: environment name",
     PERMISSIONS),
]


def make_hook(filename, comment, permissions):
    """Create a hook script.

    :param filename: The name of the file to write.
    :param comment: The comment to insert into the file.
    """
    filename = get_path(filename)
    if not os.path.exists(filename):
        log.info('creating %s', filename)
        f = open(filename, 'w')
        try:
            # for sourced scripts, the shebang line won't be used;
            # it is useful for editors to recognize the file type, though
            f.write("#!%(shell)s\n# %(comment)s\n\n" % {
                'comment': comment,
                'shell': os.environ.get('SHELL', '/bin/sh'),
            })
        finally:
            f.close()
        os.chmod(filename, permissions)
    return


# HOOKS


def initialize(args):
    for filename, comment, permissions in GLOBAL_HOOKS:
        make_hook(get_path('$VIRTUALENVWRAPPER_HOOK_DIR', filename),
                  comment, permissions)
    return


def initialize_source(args):
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/initialize" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/initialize"
"""


def pre_mkvirtualenv(args):
    log.debug('pre_mkvirtualenv %s', str(args))
    envname = args[0]
    for filename, comment, permissions in LOCAL_HOOKS:
        make_hook(get_path('$WORKON_HOME', envname, script_folder, filename),
                  comment, permissions)
    run_global('premkvirtualenv', *args)
    return


def post_mkvirtualenv_source(args):
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/postmkvirtualenv" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/postmkvirtualenv"
"""


def pre_cpvirtualenv(args):
    log.debug('pre_cpvirtualenv %s', str(args))
    envname = args[0]
    for filename, comment, permissions in LOCAL_HOOKS:
        make_hook(get_path('$WORKON_HOME', envname, script_folder, filename),
                  comment, permissions)
    run_global('precpvirtualenv', *args)
    return


def post_cpvirtualenv_source(args):
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/postcpvirtualenv" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/postcpvirtualenv"
"""


def pre_rmvirtualenv(args):
    log.debug('pre_rmvirtualenv')
    run_global('prermvirtualenv', *args)
    return


def post_rmvirtualenv(args):
    log.debug('post_rmvirtualenv')
    run_global('postrmvirtualenv', *args)
    return


def pre_activate(args):
    log.debug('pre_activate')
    run_global('preactivate', *args)
    script_path = get_path('$WORKON_HOME', args[0],
                           script_folder, 'preactivate')
    run_script(script_path, *args)
    return


def post_activate_source(args):
    log.debug('post_activate')
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/postactivate" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/postactivate"
[ -f "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/postactivate" ] && \
    source "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/postactivate"
"""


def pre_deactivate_source(args):
    log.debug('pre_deactivate')
    return """
#
# Run user-provided scripts
#
[ -f "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/predeactivate" ] && \
    source "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_ENV_BIN_DIR/predeactivate"
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/predeactivate" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/predeactivate"
"""


def post_deactivate_source(args):
    log.debug('post_deactivate')
    return """
#
# Run user-provided scripts
#
VIRTUALENVWRAPPER_LAST_VIRTUAL_ENV="$WORKON_HOME/%(env_name)s"
[ -f "$WORKON_HOME/%(env_name)s/bin/postdeactivate" ] && \
    source "$WORKON_HOME/%(env_name)s/bin/postdeactivate"
[ -f "$VIRTUALENVWRAPPER_HOOK_DIR/postdeactivate" ] && \
    source "$VIRTUALENVWRAPPER_HOOK_DIR/postdeactivate"
unset VIRTUALENVWRAPPER_LAST_VIRTUAL_ENV
""" % {'env_name': args[0]}


def get_env_details(args):
    log.debug('get_env_details')
    run_global('get_env_details', *args)
    script_path = get_path('$WORKON_HOME', args[0],
                           script_folder, 'get_env_details')
    run_script(script_path, *args)
    return


def get_path(*args):
    '''
    Get a full path from args.

    Path separator is determined according to the os and the shell and
    allow to use is_msys.

    Variables and user are expanded during the process.
    '''
    path = os.path.expanduser(os.path.expandvars(os.path.join(*args)))
    if is_msys:
        # MSYS accept unix or Win32 and sometimes
        # it drives to mixed style paths
        if re.match(r'^/[a-zA-Z](/|^)', path):
            # msys path could starts with '/c/'-form drive letter
            path = ''.join((path[1], ':', path[2:]))
        path = path.replace('/', os.sep)

    return os.path.abspath(path)



# encoding: utf-8
#
# Copyright (c) 2010 Doug Hellmann.  All rights reserved.
#
"""Load hooks for virtualenvwrapper.
"""

import inspect
import itertools
import logging
import logging.handlers
import optparse
import os
import sys

from stevedore import ExtensionManager
from stevedore import NamedExtensionManager

LOG_FORMAT = '%(asctime)s %(levelname)s %(name)s %(message)s'


class GroupWriteRotatingFileHandler(logging.handlers.RotatingFileHandler):
    """Taken from http://stackoverflow.com/questions/1407474
    """
    def _open(self):
        prevumask = os.umask(0o002)
        rtv = logging.handlers.RotatingFileHandler._open(self)
        os.umask(prevumask)
        return rtv


def main():
    parser = optparse.OptionParser(
        usage='usage: %prog [options] <hook> [<arguments>]',
        prog='virtualenvwrapper.hook_loader',
        description='Manage hooks for virtualenvwrapper',
    )

    parser.add_option(
        '-S', '--script',
        help='Runs "hook" then "<hook>_source", writing the ' +
        'result to <file>',
        dest='script_filename',
        default=None,
    )
    parser.add_option(
        '-s', '--source',
        help='Print the shell commands to be run in the current shell',
        action='store_true',
        dest='sourcing',
        default=False,
    )
    parser.add_option(
        '-l', '--list',
        help='Print a list of the plugins available for the given hook',
        action='store_true',
        default=False,
        dest='listing',
    )
    parser.add_option(
        '-v', '--verbose',
        help='Show more information on the console',
        action='store_const',
        const=2,
        default=1,
        dest='verbose_level',
    )
    parser.add_option(
        '-q', '--quiet',
        help='Show less information on the console',
        action='store_const',
        const=0,
        dest='verbose_level',
    )
    parser.add_option(
        '-n', '--name',
        help='Only run the hook from the named plugin',
        action='append',
        dest='names',
        default=[],
    )
    parser.disable_interspersed_args()  # stop when on option without an '-'
    options, args = parser.parse_args()

    root_logger = logging.getLogger('')

    # Set up logging to a file
    logfile = os.environ.get('VIRTUALENVWRAPPER_LOG_FILE')
    if logfile:
        root_logger.setLevel(logging.DEBUG)
        file_handler = GroupWriteRotatingFileHandler(
            logfile,
            maxBytes=10240,
            backupCount=1,
        )
        formatter = logging.Formatter(LOG_FORMAT)
        file_handler.setFormatter(formatter)
        root_logger.addHandler(file_handler)

    # Send higher-level messages to the console, too
    console = logging.StreamHandler()
    console_level = [logging.WARNING,
                     logging.INFO,
                     logging.DEBUG,
                     ][options.verbose_level]
    console.setLevel(console_level)
    formatter = logging.Formatter('%(name)s %(message)s')
    console.setFormatter(formatter)
    root_logger.addHandler(console)

    # logging.getLogger(__name__).debug('cli args %s', args)

    # Determine which hook we're running
    if not args:
        if options.listing:
            list_hooks()
            return 0
        else:
            parser.error('Please specify the hook to run')
    hook = args[0]

    if options.sourcing and options.script_filename:
        parser.error('--source and --script are mutually exclusive.')

    if options.sourcing:
        hook += '_source'

    log = logging.getLogger(__name__)

    log.debug('Running %s hooks', hook)
    run_hooks(hook, options, args)

    if options.script_filename:
        log.debug('Saving sourcable %s hooks to %s',
                  hook, options.script_filename)
        options.sourcing = True
        output = open(options.script_filename, "w")
        try:
            output.write('# %s\n' % hook)
            # output.write('echo %s\n' % hook)
            # output.write('set -x\n')
            run_hooks(hook + '_source', options, args, output)
        finally:
            output.close()

    return 0


def run_hooks(hook, options, args, output=None):
    if output is None:
        output = sys.stdout

    namespace = 'virtualenvwrapper.%s' % hook
    if options.names:
        hook_mgr = NamedExtensionManager(namespace, options.names)
    else:
        hook_mgr = ExtensionManager(namespace)

    if options.listing:
        def show(ext):
            output.write('  %-10s -- %s\n' %
                         (ext.name, inspect.getdoc(ext.plugin) or ''))
        try:
            hook_mgr.map(show)
        except RuntimeError:  # no templates
            output.write('  No templates installed.\n')

    elif options.sourcing:
        def get_source(ext, args):
            # Show the shell commands so they can
            # be run in the calling shell.
            contents = (ext.plugin(args) or '').strip()
            if contents:
                output.write('# %s\n' % ext.name)
                output.write(contents)
                output.write("\n")
        try:
            hook_mgr.map(get_source, args[1:])
        except RuntimeError:
            pass

    else:
        # Just run the plugin ourselves
        def invoke(ext, args):
            ext.plugin(args)
        try:
            hook_mgr.map(invoke, args[1:])
        except RuntimeError:
            pass


def list_hooks(output=None):
    if output is None:
        output = sys.stdout
    static_names = [
        'initialize',
        'get_env_details',
        'project.pre_mkproject',
        'project.post_mkproject',
        'project.template',
    ]
    pre_post_hooks = (
        '_'.join(h)
        for h in itertools.product(['pre', 'post'],
                                   ['mkvirtualenv',
                                    'rmvirtualenv',
                                    'activate',
                                    'deactivate',
                                    'cpvirtualenv',
                                    ])
    )
    for hook in itertools.chain(static_names, pre_post_hooks):
        output.write(hook + '\n')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8
#
# Copyright (c) 2010 Doug Hellmann.  All rights reserved.
#
"""virtualenvwrapper.project plugin for tests
"""

import logging
import os

log = logging.getLogger(__name__)


def template(args):
    """Creates a test file containing the args passed to us
    """
    print('Running test template with args %r' % args)
    project, project_dir = args
    filename = os.path.join(project_dir, 'TEST_FILE')
    print('Writing to %s' % filename)
    output = open(filename, 'w')
    try:
        output.write('\n'.join(args))
    finally:
        output.close()
    return
